@@ -34,7 +34,7 @@
 #include <Solid/Device>
 #include <Solid/DeviceNotifier>
 
-#include <kauthexecutejob.h>
+#include <KAuthExecuteJob>
 #include <KAuthAction>
 #include <KIdleTime>
 #include <KLocalizedString>
@@ -49,6 +49,12 @@
 
 #include <QDebug>
 
+#include <algorithm>
+
+#ifdef Q_OS_LINUX
+#include <sys/timerfd.h>
+#endif
+
 namespace PowerDevil
 {
 
@@ -73,6 +79,8 @@ Core::Core(QObject* parent)
         m_hasDualGpu = discreteGpuJob->data()[QStringLiteral("hasdualgpu")].toBool();
     });
     discreteGpuJob->start();
+
+    readChargeThreshold();
 }
 
 Core::~Core()
@@ -196,6 +204,28 @@ void Core::onBackendReady()
         QTimer::singleShot(30000, this, SLOT(onNotificationTimeout()));
     }
 
+#ifdef Q_OS_LINUX
+
+    // try creating a timerfd which can wake system from suspend
+    m_timerFd = timerfd_create(CLOCK_REALTIME_ALARM, TFD_CLOEXEC);
+
+    // if that fails due to privilges maybe, try normal timerfd
+    if (m_timerFd == -1) {
+        qCDebug(POWERDEVIL) << "Unable to create a CLOCK_REALTIME_ALARM timer, trying CLOCK_REALTIME\n This would mean that wakeup requests won't wake device from suspend";
+        m_timerFd = timerfd_create(CLOCK_REALTIME, TFD_CLOEXEC);
+    }
+
+    if (m_timerFd != -1) {
+        m_timerFdSocketNotifier = new QSocketNotifier(m_timerFd, QSocketNotifier::Read);
+        connect(m_timerFdSocketNotifier, &QSocketNotifier::activated, this, &Core::timerfdEventHandler);
+        // we disable events reading for now
+        m_timerFdSocketNotifier->setEnabled(false);
+    } else {
+        qCDebug(POWERDEVIL) << "Unable to create a CLOCK_REALTIME timer, scheduled wakeups won't be available";
+    }
+
+#endif
+
     // All systems up Houston, let's go!
     Q_EMIT coreReady();
     refreshStatus();
@@ -240,6 +270,8 @@ void Core::reparseConfiguration()
     if (m_lowBatteryNotification && currentChargePercent() > PowerDevilSettings::batteryLowLevel()) {
         m_lowBatteryNotification->close();
     }
+
+    readChargeThreshold();
 }
 
 QString Core::currentProfile() const
@@ -905,6 +937,36 @@ void Core::onServiceRegistered(const QString &service)
     }
 }
 
+void Core::readChargeThreshold()
+{
+    KAuth::Action action(QStringLiteral("org.kde.powerdevil.chargethresholdhelper.getthreshold"));
+    action.setHelperId(QStringLiteral("org.kde.powerdevil.chargethresholdhelper"));
+    KAuth::ExecuteJob *job = action.execute();
+    connect(job, &KJob::result, this, [this, job] {
+        if (job->error()) {
+            qCWarning(POWERDEVIL) << "org.kde.powerdevil.chargethresholdhelper.getthreshold failed" << job->errorText();
+            return;
+        }
+
+        const auto data = job->data();
+
+        const int chargeStartThreshold = data.value(QStringLiteral("chargeStartThreshold")).toInt();
+        if (m_chargeStartThreshold != chargeStartThreshold) {
+            m_chargeStartThreshold = chargeStartThreshold;
+            Q_EMIT chargeStartThresholdChanged(chargeStartThreshold);
+        }
+
+        const int chargeStopThreshold = data.value(QStringLiteral("chargeStopThreshold")).toInt();
+        if (m_chargeStopThreshold != chargeStopThreshold) {
+            m_chargeStopThreshold = chargeStopThreshold;
+            Q_EMIT chargeStopThresholdChanged(chargeStopThreshold);
+        }
+
+        qCDebug(POWERDEVIL) << "Charge thresholds: start at" << chargeStartThreshold << "- stop at" << chargeStopThreshold;
+    });
+    job->start();
+}
+
 BackendInterface* Core::backend()
 {
     return m_backend;
@@ -925,6 +987,63 @@ bool Core::hasDualGpu() const
     return m_hasDualGpu;
 }
 
+int Core::chargeStartThreshold() const
+{
+    return m_chargeStartThreshold;
+}
+
+int Core::chargeStopThreshold() const
+{
+    return m_chargeStopThreshold;
+}
+
+uint Core::scheduleWakeup(const QString &service, const QDBusObjectPath &path, qint64 timeout)
+{
+    ++m_lastWakeupCookie;
+
+    int cookie = m_lastWakeupCookie;
+    // if some one is trying to time travel, deny them
+    if (timeout < QDateTime::currentSecsSinceEpoch()) {
+        sendErrorReply(QDBusError::InvalidArgs, "You can not schedule wakeup in past");
+    } else {
+#ifndef Q_OS_LINUX
+        sendErrorReply(QDBusError::NotSupported, "Scheduled wakeups are available only on Linux platforms");
+#else
+        WakeupInfo wakeup{ service, path, cookie, timeout };
+        m_scheduledWakeups << wakeup;
+        qCDebug(POWERDEVIL) << "Received request to wakeup at " << QDateTime::fromSecsSinceEpoch(timeout);
+        resetAndScheduleNextWakeup();
+#endif
+    }
+    return cookie;
+}
+
+void Core::wakeup()
+{
+    onResumingFromIdle();
+}
+
+void Core::clearWakeup(int cookie)
+{
+    // if we do not have any timeouts return from here
+    if (m_scheduledWakeups.isEmpty()) {
+        return;
+    }
+
+    // depending on cookie, remove it from scheduled wakeups
+    auto erased = m_scheduledWakeups.erase(std::remove_if(m_scheduledWakeups.begin(), m_scheduledWakeups.end(), [cookie](WakeupInfo wakeup) {
+        return wakeup.cookie == cookie;
+    }));
+
+    if (erased == m_scheduledWakeups.end()) {
+        sendErrorReply(QDBusError::InvalidArgs, "Can not clear the invalid wakeup");
+        return;
+    }
+
+    // reset timerfd
+    resetAndScheduleNextWakeup();
+}
+
 qulonglong Core::batteryRemainingTime() const
 {
     return m_backend->batteryRemainingTime();
@@ -935,4 +1054,63 @@ uint Core::backendCapabilities()
     return m_backend->capabilities();
 }
 
+void Core::resetAndScheduleNextWakeup()
+{
+
+#ifdef Q_OS_LINUX
+    // first we sort the wakeup list
+    std::sort(m_scheduledWakeups.begin(), m_scheduledWakeups.end(), [](const WakeupInfo& lhs, const WakeupInfo& rhs) 
+    {
+        return lhs.timeout < rhs.timeout;
+    });
+
+    // we don't want any of our wakeups to repeat'
+    timespec interval = {0, 0};
+    timespec nextWakeup;
+    bool enableNotifier = false;
+    // if we don't have any wakeups left, we call it a day and stop timer_fd
+    if(m_scheduledWakeups.isEmpty()) {
+        nextWakeup = {0, 0};
+    } else {
+        // now pick the first timeout from the list
+        WakeupInfo wakeup = m_scheduledWakeups.first();
+        nextWakeup = {wakeup.timeout, 0};
+        enableNotifier = true;
+    }
+    if (m_timerFd != -1) {
+        const itimerspec spec = {interval, nextWakeup};
+        timerfd_settime(m_timerFd, TFD_TIMER_ABSTIME, &spec, nullptr);
+    }
+    m_timerFdSocketNotifier->setEnabled(enableNotifier);
+#endif
+}
+
+void Core::timerfdEventHandler()
+{
+    // wakeup from the linux/rtc
+
+    // Disable reading events from the timer_fd
+    m_timerFdSocketNotifier->setEnabled(false);
+
+    // At this point scheduled wakeup list should not be empty, but just in case
+    if (m_scheduledWakeups.isEmpty()) {
+        qWarning(POWERDEVIL) << "Wakeup was recieved but list is now empty! This should not happen!";
+        return;
+    }
+
+    // first thing to do is, we pick the first wakeup from list
+    WakeupInfo currentWakeup = m_scheduledWakeups.takeFirst();
+
+    // Before doing anything further, lets set the next set of wakeup alarm
+    resetAndScheduleNextWakeup();
+
+    // Now current wakeup needs to be processed
+    // prepare message for sending back to the consumer
+    QDBusMessage msg = QDBusMessage::createMethodCall(currentWakeup.service, currentWakeup.path.path(),
+                                                      QStringLiteral("org.kde.PowerManagement"), QStringLiteral("wakeupCallback"));
+    msg << currentWakeup.cookie;
+    // send it away
+    QDBusConnection::sessionBus().call(msg, QDBus::NoBlock);
+}
+
 }
